package ac.ed.lurg.demand;

import java.util.HashMap;
import java.util.Map;

import ac.ed.lurg.ModelConfig;
import ac.ed.lurg.Timestep;
import ac.ed.lurg.country.CompositeCountry;
import ac.ed.lurg.country.CompositeCountryManager;
import ac.ed.lurg.country.GlobalPrice;
import ac.ed.lurg.country.SingleCountry;
import ac.ed.lurg.types.CommodityType;
import ac.ed.lurg.types.CropType;
import ac.ed.lurg.utils.LogWriter;

public abstract class AbstractDemandManager {

	protected CompositeCountryManager compositeCountryManager;
	protected BioenergyDemandManager bioenergyDemandManager;

	public AbstractDemandManager(CompositeCountryManager compositeCountryManager) {
		this.compositeCountryManager = compositeCountryManager;
		bioenergyDemandManager = new BioenergyDemandManager();
	}

	public Map<CommodityType, Double> getDemand(CompositeCountry cc, int year) {

		if (!ModelConfig.CHANGE_DEMAND_YEAR)
			year = ModelConfig.BASE_YEAR;

		Map<CommodityType, Double> demandMap = new HashMap<CommodityType, Double>();
		Map<CommodityType, Double> foodDemandMap;

		for (SingleCountry c : compositeCountryManager.getAllForCompositeCountry(cc)) {

			if (ModelConfig.ACTIVE_DIETARY_SHIFT) {
				foodDemandMap = calculateDietaryShift(c, year);
			} else
				foodDemandMap = getFoodDemand(c, year);

			for (CommodityType commodity : CommodityType.getAllItems()) {

				Double food = foodDemandMap.get(commodity);
				double bioenergy = bioenergyDemandManager.getFirstGenBioenergyDemand(c, year, commodity);
				// LogWriter.println(String.format("Country %s comm %s: %f",
				// country, commodity, bioenergy));

				double newDemand = ((food == null ? 0.0 : food.doubleValue()) + bioenergy)
						* (1.0 + commodity.getSeedWasteRate());
				Double prevDemand = demandMap.get(commodity);

				if (prevDemand != null)
					newDemand += prevDemand.doubleValue(); // aggregate if
															// composite country

				demandMap.put(commodity, newDemand);
			}
		}
		return demandMap;
	}

	abstract Map<CommodityType, Double> getFoodDemand(SingleCountry c, int year);

	public double getSecondGenBioenergyDemand(Timestep timestep) {
		int year = ModelConfig.CHANGE_DEMAND_YEAR ? timestep.getYear() : ModelConfig.BASE_YEAR;
		double d = bioenergyDemandManager.getSecondGenBioenergyDemand(year);
		LogWriter.println("Global gen2 bioenergy demand in " + timestep + " is " + d);
		return d;
	}

	private Map<CommodityType, Double> calculateDietaryShift(SingleCountry c, int year) {

		int yearsOfChange = Math.min(ModelConfig.DIETARY_CHANGE_END_YEAR - ModelConfig.DIETARY_CHANGE_START_YEAR,
				year - ModelConfig.DIETARY_CHANGE_START_YEAR);
		double dietaryDemandAdj = yearsOfChange > 0 ? (yearsOfChange * ModelConfig.DIETARY_CHANGE_ANNUAL_RATE) : 0.0;

		Map<CommodityType, Double> originalFoodDemandMap = getFoodDemand(c, year);

		Double monogastricDemand = originalFoodDemandMap.get(CommodityType.MONOGASTRICS);
		double mongoastricCalorieShortfall = (monogastricDemand != null)
				? monogastricDemand * dietaryDemandAdj * CommodityType.MONOGASTRICS.getEnergyPerTon() : 0.0;

		Double ruminantDemand = originalFoodDemandMap.get(CommodityType.RUMINANTS);
		double ruminantsCalorieShortfall = (ruminantDemand != null)
				? ruminantDemand * dietaryDemandAdj * CommodityType.RUMINANTS.getEnergyPerTon() : 0.0;

		Map<CommodityType, Double> updatedFoodDemandMap = new HashMap<CommodityType, Double>();

		for (CommodityType commodity : CommodityType.getAllItems()) {
			Double originalCommodityDemand = originalFoodDemandMap.get(commodity);

			if (originalCommodityDemand != null) {
				if (commodity.isAnimalProduct()) {
					updatedFoodDemandMap.put(commodity,
							originalCommodityDemand - originalCommodityDemand * dietaryDemandAdj);
				} else {
					updatedFoodDemandMap.put(commodity,
							originalCommodityDemand
									+ commodity.getMeatSubstitutionProportion() * ruminantsCalorieShortfall
											/ commodity.getEnergyPerTon()
									+ commodity.getMeatSubstitutionProportion() * mongoastricCalorieShortfall
											/ commodity.getEnergyPerTon());

				}
			}

		}

		//LogWriter.println("demand adj " + dietaryDemandAdj + " year " + year + " years of change " + yearsOfChange );
		//LogWriter.println("original map " + originalFoodDemandMap);
		//LogWriter.println("updated map " + updatedFoodDemandMap);
		return updatedFoodDemandMap;
	}

}
